@@ -37,7 +37,7 @@ from colabo_flow.s_go.operators import createGoMsg
 from colabo_flow.s_go.executor_vos import ITaskExecutor, IFlowTask
 from colabo_flow.s_go.executor_operators import createTaskInstanceForTaskWithoutFlow
 
-from colabo_flow.s_go.concurrency import async_run_func_under_thread, run_func_under_thread, async_connection_callback_threadsafe
+from colabo_flow.s_go.concurrency import run_async_func_under_thread, async_connection_callback_threadsafe
 LANGUAGE = EProgrammingLanguageSupported.PYTHON
 
 def host_registerTaskExecutor(host: IColaboFlowHostData, executor: ITaskExecutor):
@@ -139,7 +139,7 @@ def _basic_publish(host: IColaboFlowHostData, sendToQueue: str, msg: IColaboFlow
 	msgStr = json.dumps(msg)
 	host.chHostHeartbeat.basic_publish(exchange='', routing_key=sendToQueue, body=msgStr)
 
-async def processColaboFlowMsg_TaskExecute(host: IColaboFlowHostData, msgServiceReqObj: IColaboFlowGoMsg):
+async def async_threaded_processColaboFlowMsg_TaskExecute(host: IColaboFlowHostData, msgServiceReqObj: IColaboFlowGoMsg):
 
 	msgServiceTaskExecuteReqObjExtension: IColaboFlowGoMsgExtension_TaskExecute_Request = from_dict(data_class=IColaboFlowGoMsgExtension_TaskExecute_Request, data=msgServiceReqObj.extension)
 
@@ -149,14 +149,18 @@ async def processColaboFlowMsg_TaskExecute(host: IColaboFlowHostData, msgService
 	if not queueReplyTo:
 		queueReplyTo = host.queueHostHeartbeat
 
+	# *************************
+	# RETRIEVING THE TASK and TASK EXECUTOR
+	# *************************/
+
 	task: IFlowTask = host.tasksAvailable[taskId]
 
-	print(f'{host.appNameTextWithContext("processColaboFlowMsg_TaskExecute")} TASK EXECUTION REQUEST: taskId: {chalk.blue_bright(taskId)}, queueReplyTo: "{queueReplyTo}"')
+	print(f'{host.appNameTextWithContext("async_threaded_processColaboFlowMsg_TaskExecute")} TASK EXECUTION REQUEST: taskId: {chalk.blue_bright(taskId)}, queueReplyTo: "{queueReplyTo}"')
 
 	if not task:
-		print(f'{host.appNameTextWithContext("processColaboFlowMsg_TaskExecute")} task is missing for the id: {taskId}')
+		print(f'{host.appNameTextWithContext("async_threaded_processColaboFlowMsg_TaskExecute")} task is missing for the id: {taskId}')
 
-	print(f'{host.appNameTextWithContext("processColaboFlowMsg_TaskExecute")} Executing id: {chalk.blue_bright(taskId)}, task.description: {task.description}')
+	print(f'{host.appNameTextWithContext("async_threaded_processColaboFlowMsg_TaskExecute")} Executing id: {chalk.blue_bright(taskId)}, task.description: {task.description}')
 
 	# create task instance
 	taskInstanceId: str = f'taskId:lang-{LANGUAGE}:host-{host.hostName}:{uuid.uuid1()}'
@@ -170,11 +174,11 @@ async def processColaboFlowMsg_TaskExecute(host: IColaboFlowHostData, msgService
 
 	# get the task executor
 	if not task.executorRef or not task.executorRef.id:
-		print(f'{host.appNameTextWithContext("processColaboFlowMsg_TaskExecute")} Executor ID is not provided for the task: {taskId}')
+		print(f'{host.appNameTextWithContext("async_threaded_processColaboFlowMsg_TaskExecute")} Executor ID is not provided for the task: {taskId}')
 		return
 
 	taskExecutor: ITaskExecutor = getTaskExecutor(host, task.executorRef.id)
-	print(f'{host.appNameTextWithContext("processColaboFlowMsg_TaskExecute")} taskExecutor: {taskExecutor}')
+	print(f'{host.appNameTextWithContext("async_threaded_processColaboFlowMsg_TaskExecute")} taskExecutor: {taskExecutor}')
 
 	# *************************
 	# REPLYING THE TASK PROGRESS (task instance id)
@@ -190,79 +194,31 @@ async def processColaboFlowMsg_TaskExecute(host: IColaboFlowHostData, msgService
 	msgTaskExecutionProgressDict: Any = dataclasses.asdict(msgTaskExecutionProgress)
 
 	if host.reportsConfig.showExecutionMsgs:
-		print(f'{host.appNameTextWithContext("processColaboFlowMsg_TaskExecute")} Sending PROGRESS to the task execution requester (service) [queue: "{chalk.blue_bright(queueReplyTo)}"]: {json.dumps(msgTaskExecutionProgressDict)}')
+		print(f'{host.appNameTextWithContext("async_threaded_processColaboFlowMsg_TaskExecute")} Sending PROGRESS to the task execution requester (service) [queue: "{chalk.blue_bright(queueReplyTo)}"]: {json.dumps(msgTaskExecutionProgressDict)}')
 
-	await async_connection_callback_threadsafe(host.connection, host.appNameTextWithContext, lambda: _basic_publish(host, queueReplyTo, msgTaskExecutionProgressDict))
+	await async_connection_callback_threadsafe(host.connection, host.appNameTextWithContext, lambda: _basic_publish(host, queueReplyTo, msgTaskExecutionProgressDict), f"_basic_publish:msgTaskExecutionProgressDict")
 
-	# host.connection.add_callback_threadsafe(lambda: _basic_publish(host, queueReplyTo, msgTaskExecutionProgressDict))
-	# host.chHostHeartbeat.basic_publish(exchange='', routing_key=queueReplyTo, body=msgTaskExecutionProgressDict)
-	# # host.connection.add_callback_threadsafe(
-	# 	# functools.partial(host.chHostHeartbeat.basic_publish, exchange='', routing_key=host.queueHostHeartbeat, body=heartbeat.to_json()))
+	# *************************
+	# EXECUTING THE TASK (task instance id)
+	# *************************/
+	print(f'{host.appNameTextWithContext("async_threaded_processColaboFlowMsg_TaskExecute")} Executing task with taskId "{chalk.blue.bold(taskId)}"')
 
-	def _executeTask(loop, futTaskExecuted, taskExecutor: ITaskExecutor, taskId: str, taskInput: Any):
-		# *************************
-		# EXECUTING THE TASK
-		# *************************/
+	if taskExecutor.reference.async_method:
+		print(f'{host.appNameTextWithContext("async_threaded_processColaboFlowMsg_TaskExecute")} executing async executor (taskExecutor.reference.async_method)')
+		taskOutput = await taskExecutor.reference.async_method(taskInput)
+	elif taskExecutor.reference.method:
+		print(f'{host.appNameTextWithContext("async_threaded_processColaboFlowMsg_TaskExecute")} executing sync executor (taskExecutor.reference.method)')
+		taskOutput = taskExecutor.reference.method(taskInput)
+	else:
+		errorMsg = f'{host.appNameTextWithContext("async_threaded_processColaboFlowMsg_TaskExecute")} Error, neither async nor non-async executor reference available for taskId: {taskId} and executorId: "{taskExecutor.id}"'
+		print(errorMsg)
+		raise Exception(errorMsg)
 
-		""" task execution output """
-		taskOutput: Any = None
-
-		async def async_runTask(async_method, taskInput: Any):
-			global taskOutput
-			taskOutput = await async_method(taskInput)
-
-		_asyncio_task_loop = asyncio.new_event_loop()
-		asyncio.set_event_loop(_asyncio_task_loop)
-
-		print(f'{host.appNameTextWithContext("_executeTask")}  event_loop: {_asyncio_task_loop}, Executing task with taskId "{chalk.blue.bold(taskId)}"')
-		if taskExecutor.reference.async_method:
-			print(f'{host.appNameTextWithContext("processColaboFlowMsg_TaskExecute")} executing async executor (taskExecutor.reference.async_method)')
-			# taskOutput = await taskExecutor.reference.async_method(taskInput)
-			asyncio.run(async_runTask(taskExecutor.reference.async_method, taskInput))
-		elif taskExecutor.reference.method:
-			print(f'{host.appNameTextWithContext("processColaboFlowMsg_TaskExecute")} executing sync executor (taskExecutor.reference.method)')
-			taskOutput = taskExecutor.reference.method(taskInput)
-		else:
-			errorMsg = f'{host.appNameTextWithContext("processColaboFlowMsg_TaskExecute")} Error, neither async nor non-async executor reference available for taskId: {taskId} and executorId: "{taskExecutor.id}"'
-			print(errorMsg)
-			raise Exception(errorMsg)
-
-		print(f'{host.appNameTextWithContext("_executeTask")} Task with taskId "{chalk.blue.bold(taskId)}" executed. taskOutput: {taskOutput}')
-		# futLambdaExecuted.set_result(True)
-		loop.call_soon_threadsafe(futTaskExecuted.set_result, taskOutput)
-
-	async def executeTask(taskExecutor: ITaskExecutor, taskId: str, taskInput: Any):
-
-		# Get the current event loop.
-		loop = asyncio.get_running_loop()
-
-		# Create a new Future object.
-		futTaskExecuted = loop.create_future()
-
-		try:
-			print(f'{host.appNameTextWithContext("executeTask")} Calling "_executeTask" thread')
-			# _executeTask(loop, futTaskExecuted, taskExecutor, taskId, taskInput )
-			taskExecutionThread = threading.Thread(target=_executeTask, args=(loop, futTaskExecuted, taskExecutor, taskId, taskInput ))
-			print(f'{host.appNameTextWithContext("executeTask")} Called "_executeTask" thread')
-		except Error:
-			print(f'{host.appNameTextWithContext("executeTask")} Error creating the "_executeTask" thread')
-
-		taskExecutionThread.name = f'{taskExecutionThread.name}_executeTask_taskId-{taskId}'
-		taskExecutionThread.start()
-		native_id = taskExecutionThread.native_id
-		print(f'{host.appNameTextWithContext("executeTask")} the "_executeTask" thread (with the native native_id: {chalk.blue.bold(native_id)} ) is spawned')
-
-		# taskOutput = None
-		""" task execution output """
-		taskOutput: Any = await futTaskExecuted
-		print(f'{host.appNameTextWithContext("executeTask")} Task with taskId "{chalk.blue.bold(taskId)}" executed. taskOutput: {taskOutput}')
-
-		return taskOutput
-
-	taskOutput = await executeTask(taskExecutor, taskId, taskInput)
-
-	print(f'{host.appNameTextWithContext("processColaboFlowMsg_TaskExecute")} Task with taskId "{chalk.blue.bold(taskId)}" executed. Result is: {chalk.blue_bright(taskOutput)}')
+	print(f'{host.appNameTextWithContext("async_threaded_processColaboFlowMsg_TaskExecute")} Task with taskId "{chalk.blue.bold(taskId)}" executed. Result is: {chalk.blue_bright(taskOutput)}')
 
+	# *************************
+	# SENDING EXECUTION RESPONSE
+	# *************************/
 	msgTaskExecutionReply: IColaboFlowGoMsg = createGoMsg(host.hostName, EColaboFlowGoMsgType.TASK_EXECUTE_RESPONSE)
 	msgTaskExecutionReply.requestId = msgServiceReqObj.requestId
 	msgTaskExecutionReply.msgReplyChainIds = [msgServiceReqObj.id]
@@ -274,12 +230,12 @@ async def processColaboFlowMsg_TaskExecute(host: IColaboFlowHostData, msgService
 	msgTaskExecutionProgressDict: Any = dataclasses.asdict(msgTaskExecutionProgress)
 
 	if host.reportsConfig.showExecutionMsgs:
-		print(f'{host.appNameTextWithContext("processColaboFlowMsg_TaskExecute")} Sending RESPONSE to the task execution requester (service) [queue: "{chalk.blue_bright(queueReplyTo)}"]: {msgTaskExecutionReply}')
+		print(f'{host.appNameTextWithContext("async_threaded_processColaboFlowMsg_TaskExecute")} Sending RESPONSE to the task execution requester (service) [queue: "{chalk.blue_bright(queueReplyTo)}"]: {msgTaskExecutionReply}')
 
 	msgTaskExecutionReplyDict: Any = dataclasses.asdict(msgTaskExecutionReply)
-	await async_connection_callback_threadsafe(host.connection, host.appNameTextWithContext, lambda: _basic_publish(host, queueReplyTo, msgTaskExecutionReplyDict))
+	await async_connection_callback_threadsafe(host.connection, host.appNameTextWithContext, lambda: _basic_publish(host, queueReplyTo, msgTaskExecutionReplyDict), f"_basic_publish:msgTaskExecutionReplyDict")
 
-def processColaboFlowMsg_TaskInstancesReport(host: IColaboFlowHostData, msg: IColaboFlowGoMsg):
+def async_processColaboFlowMsg_TaskInstancesReport(host: IColaboFlowHostData, msg: IColaboFlowGoMsg):
 	pass
 
 def listenOnHostQueue(host: IColaboFlowHostData):
@@ -287,40 +243,46 @@ def listenOnHostQueue(host: IColaboFlowHostData):
 	print(f'{host.appNameTextWithContext("listenOnHostQueue")} Listening on the queue: "{chalk.blue_bright(host.queueHost)}"')
 	print(f'{host.appNameTextWithContext("listenOnHostQueue")} [*] Waiting for messages in %s. To exit press "{chalk.blue_bright("CTRL+C")}"')
 
+	async def async_threaded_callback_host_msg_request(ch, method, properties, body):
+		# print(f"{host.appNameTextWithContext('async_threaded_callback_host_msg_request')} Received {body}")
+		msgObj: Any = json.loads(body)
+		print(f"{host.appNameTextWithContext('async_threaded_callback_host_msg_request')} Received {msgObj}")
+		# https://github.com/konradhalas/dacite#casting
+		msg: IColaboFlowGoMsg = from_dict(data_class=IColaboFlowGoMsg, data=msgObj,
+			config=Config(cast=[EColaboFlowGoMsgType])
+		)
+		print(f'{host.appNameTextWithContext("async_threaded_callback_host_msg_request")} msg.sender: "{chalk.blue_bright(msg.sender)}", msg.type: "{chalk.blue_bright(msg.type)}, msg.msg: "{chalk.blue_bright(msg.msg)}"')
+
+		# print(f'{host.appNameTextWithContext("async_threaded_callback_host_msg_request")} futTaskExecute: "{futTaskExecute}"')
+		# # set the `futTaskExecute` result in the context of the main-thread
+		# loop.call_soon_threadsafe(async_threaded_callback_host_msg_request_set_result, futTaskExecute, msg["extension"]["taskOutput"])
+
+		if msg.type == EColaboFlowGoMsgType.TASK_EXECUTE_REQUEST:
+			if host.processOrchestratorTaskExecuteRequests:
+				await async_threaded_processColaboFlowMsg_TaskExecute(host, msg)
+
+		elif msg.type == EColaboFlowGoMsgType.TASK_INSTANCES_REPORT__REQUEST:
+			await async_processColaboFlowMsg_TaskInstancesReport(host, msg)
+
+		# acknowledging the received message
+		# https://www.rabbitmq.com/tutorials/tutorial-six-python.html
+		# https://github.com/rabbitmq/rabbitmq-tutorials/blob/main/python/rpc_server.py
+		print(f'{host.appNameTextWithContext("async_threaded_callback_host_msg_request")} msg.sender: "{chalk.blue_bright(msg.sender)}", Acknowledging back (msg.type: "{chalk.blue_bright(msg.type)}, method.delivery_tag: {chalk.blue_bright(method.delivery_tag)})')
+		await async_connection_callback_threadsafe(host.connection, host.appNameTextWithContext, lambda: ch.basic_ack(delivery_tag=method.delivery_tag), f"async_threaded_async_threaded_callback_host_msg_request:basic_ack")
+		print(f'{host.appNameTextWithContext("async_threaded_callback_host_msg_request")} msg is acknowledged back')
+
 	def callback_host_msg_request(ch, method, properties, body):
 		"""
 		A callback invited when a message with host request from the ColaboFlow service is received
 
 		NOTE: It runs in the context of a new connection-thread (`host.consumingThread`)
+
+		As we cannot conveniently run asyncio from it, and we constantly under a risk of blocking the connection-thread we are spawning the real business logic through an async function `async_threaded_callback_host_msg_request` under a separate thread
 		"""
-		# print(f"{host.appNameTextWithContext('callback_host_msg_request')} Received {body}")
-		msgObj: Any = json.loads(body)
-		print(f"{host.appNameTextWithContext('callback_host_msg_request')} Received {msgObj}")
-		# https://github.com/konradhalas/dacite#casting
-		msg: IColaboFlowGoMsg = from_dict(data_class=IColaboFlowGoMsg, data=msgObj,
-			config=Config(cast=[EColaboFlowGoMsgType])
-		)
-		print(f'{host.appNameTextWithContext("callback_host_msg_request")} msg.sender: "{chalk.blue_bright(msg.sender)}", msg.type: "{chalk.blue_bright(msg.type)}, msg.msg: "{chalk.blue_bright(msg.msg)}"')
 
-		# print(f'{host.appNameTextWithContext("callback_host_msg_request")} futTaskExecute: "{futTaskExecute}"')
-		# # set the `futTaskExecute` result in the context of the main-thread
-		# loop.call_soon_threadsafe(callback_host_msg_request_set_result, futTaskExecute, msg["extension"]["taskOutput"])
+		print(f'{host.appNameTextWithContext("callback_host_msg_request")} Received a new message, launching `async_threaded_callback_host_msg_request` in a separate thread')
 
-		if msg.type == EColaboFlowGoMsgType.TASK_EXECUTE_REQUEST:
-			if host.processOrchestratorTaskExecuteRequests:
-				run_func_under_thread(host.appNameTextWithContext, lambda: processColaboFlowMsg_TaskExecute(host, msg), "processColaboFlowMsg_TaskExecute")
-				# asyncio.run(
-				# 	processColaboFlowMsg_TaskExecute(host, msg)
-				# )
-
-		elif msg.type == EColaboFlowGoMsgType.TASK_INSTANCES_REPORT__REQUEST:
-			processColaboFlowMsg_TaskInstancesReport(host, msg)
-
-		# https://www.rabbitmq.com/tutorials/tutorial-six-python.html
-		# https://github.com/rabbitmq/rabbitmq-tutorials/blob/main/python/rpc_server.py
-		print(f'{host.appNameTextWithContext("callback_host_msg_request")} msg.sender: "{chalk.blue_bright(msg.sender)}", Acknowledging back (msg.type: "{chalk.blue_bright(msg.type)}, method.delivery_tag: {chalk.blue_bright(method.delivery_tag)})')
-		ch.basic_ack(delivery_tag=method.delivery_tag)
-		print(f'{host.appNameTextWithContext("callback_host_msg_request")} msg is acknowledged back')
+		run_async_func_under_thread(host.appNameTextWithContext, lambda: async_threaded_callback_host_msg_request(ch, method, properties, body), "async_threaded_callback_host_msg_request")
 
 	# Note: prefetch is set to 1 here as an example only and to keep the number of threads created
 	# to a reasonable amount. In production you will want to test with different prefetch values
